@@ -1,10 +1,12 @@
 use std::ffi::CString;
-use std::mem;
 use std::os::raw::c_char;
 use uuid::Uuid;
 
+/// The returned char pointer should be freed by the `cstring_free` function.
+/// For detailed usage in any language, please see:
+/// https://jakegoulding.com/rust-ffi-omnibus/string_return/
 #[no_mangle]
-pub extern "C" fn generate(hyphenated: bool, urn: bool, uppercase: bool) -> *const c_char {
+pub extern "C" fn generate(hyphenated: bool, urn: bool, uppercase: bool) -> *mut c_char {
     let uuid = Uuid::new_v4();
     let output = CString::new(if hyphenated {
         let uuid = uuid.as_hyphenated();
@@ -29,9 +31,21 @@ pub extern "C" fn generate(hyphenated: bool, urn: bool, uppercase: bool) -> *con
         }
     })
     .unwrap();
-    let ptr = output.as_ptr();
-    mem::forget(output);
-    ptr
+    output.into_raw()
+}
+
+/// # Safety
+///
+/// This function is for freeing a pointer of the argument.
+/// The pointer should be allocated with `CString` in the Rust world.
+#[no_mangle]
+pub unsafe extern "C" fn cstring_free(s: *mut c_char) {
+    if s.is_null() {
+        return;
+    }
+
+    // retake pointer to free memory
+    let _ = CString::from_raw(s);
 }
 
 #[cfg(test)]
@@ -46,9 +60,9 @@ mod tests {
             #include <string.h>
 
             int main() {
-                const char* output = generate(false, false, false);
+                char* output = generate(false, false, false);
                 printf("%lu", strlen(output));
-                free((char*)output);
+                cstring_free(output);
                 return 0;
             }
         })
@@ -63,9 +77,9 @@ mod tests {
             #include <string.h>
 
             int main() {
-                const char* output = generate(true, false, false);
+                char* output = generate(true, false, false);
                 printf("%lu", strlen(output));
-                free((char*)output);
+                cstring_free(output);
                 return 0;
             }
         })
@@ -80,9 +94,9 @@ mod tests {
             #include <string.h>
 
             int main() {
-                const char* output = generate(false, true, false);
+                char* output = generate(false, true, false);
                 printf("%lu", strlen(output));
-                free((char*)output);
+                cstring_free(output);
                 return 0;
             }
         })
